<?php

namespace App\Jobs;

use App\Models\Server;
use Carbon\Carbon;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;

class CheckServers implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        // We grab the server that are marked online but didn't do a heartbeat in the last two minutes.
        // This happens when a server crashes or goes offline.
        Server::where("online", true)->where('last_heartbeat_at', '<', Carbon::now()->subSeconds(120)->toDateTimeString())->lazyById()->each(function ($server) {
            // We fetch all the users connected to the server and mark them as disconnected.
            // NOTE: The users will have to wait at least one minute and at most three minutes
            // since the server crashes before connecting to another server.
            $server->users->each(function ($user) {
                $user->disconnect();
            });

            // We fetch all the game that were running, detach their users (thus deleting the pivot data)
            // and delete the game. This makes it as the game never happened.
            $server->games->whereIn("state", ["created", "playing"])->each(function($game) {
                $game->users()->detach();
                $game->delete();
            });

            // We mark the server as offline
            $server->offline();
        });
    }
}
