<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Builder;

/**
 * @OA\Schema(
 *   title="Game",
 *   type="object",
 *   @OA\Property(type="integer", property="id"),
 *   @OA\Property(type="integer", property="server_id"),
 *   @OA\Property(type="string", property="created_at", format="date-time"),
 *   @OA\Property(type="string", property="updated_at", format="date-time"),
 * )
 */
class Game extends Model
{
    use HasFactory;

    protected static function booted(): void
    {
        /*
        static::addGlobalScope('finished', function (Builder $builder) {
            $builder->where("state", "finished");
        });
        */
    }

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'server_id',
    ];

    protected $hidden = [
        'state'
    ];

    public function users()
    {
        return $this->belongsToMany(user::class, 'users_games', 'game_id', 'user_id')->using(UserGame::class);
    }

    public function server(): BelongsTo
    {
        return $this->belongsTo(Server::class, 'server_id');
    }

    public function start()
    {
        $this->started_at = Carbon::now();
        $this->state = 'playing';
        $this->save();
    }

    public function end($user_data)
    {
        $this->finished_at = Carbon::now();
        $this->state = 'finished';
        $this->save();
    }
}
